#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : barkup
# @Time         : 2024/1/9 09:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/openai/openai-python
# todo: 设计更加通用的兜底方案【首先得有靠谱的渠道（多个兜底渠道，作双兜底）】
# 3.5 deepseek
# 遇到内容审核【只要抛错就走备用】

from meutils.pipe import *
from meutils.notice.feishu import send_message

from openai import OpenAI
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error, completion_keys
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

send_message = partial(
    send_message,
    title="ChatCompletion主备",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


@lru_cache()
class Completions(object):
    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None, **kwargs):
        self.clients = []

        # 需要兜底的模型
        params = dict(
            api_key=api_key or 'sk-...',
            base_url=base_url or 'https://api.githubcopilot.com',
            default_headers={'Editor-Version': 'vscode/1.85.1'},
            timeout=300,
        )
        # 如果这一步就报错呢
        self.client = OpenAI(**params)  # todo: 异步

        # todo: 一主多备
        self.backup_client = OpenAI(
            api_key=os.getenv("BACKUP_API_KEY"),
            base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1"),  # todo: 避免自己调用自己
            timeout=300,
        )

    def create(self, request: ChatCompletionRequest, **kwargs):
        # data = request.model_dump()
        # data = {key: data.get(key) for key in completion_keys if key in data}  # 去除多余key， 可能最新的0125不支持

        data = to_openai_completion_params(request)

        # 直接走官转
        if self.check_model(request):
            send_message(f"直接走官转：{data}")
            return self._backup_create(**data)

        creates = [self.client.chat.completions.create, self._backup_create, ]
        e = None
        for i, _create in enumerate(creates):
            try:
                response = _create(**data)  # 尝试执行操作

                # break  # 如果操作成功，则跳出循环
                if i == 0 and data.get('stream'):
                    return self.post_process(response, data)

                if data.get('stream'):
                    return response
                else:
                    n = 1.2
                    response.usage.total_tokens = int(response.usage.total_tokens * n)
                    response.usage.prompt_tokens = int(response.usage.prompt_tokens * n)
                    response.usage.completion_tokens = int(response.usage.completion_tokens * n)
                    return response

            except Exception as e:  # 走兜底
                _ = f"CompletionsClient {i} failed: {e}"  # rate limit exceeded：阶段性跳过这样的请求，防止被封
                send_message(_)
                logging.error(_)
        return self._handle_error(data, e)  # UnboundLocalError: local variable 'e' referenced before assignment

    def post_process(self, response, data):  # copilot
        """兜底判断"""
        for chunk in response:
            if chunk.choices:
                if chunk.choices[0].finish_reason == 'content_filter':  # 走兜底  "finish_reason":"content_filter"
                    _ = f"ContentFilter：{chunk.model_dump_json()}"
                    logger.debug(_)
                    send_message(_)
                    yield from self._backup_create(**data)
                    break

                if chunk.choices[0].delta.content or chunk.choices[0].finish_reason:
                    yield chunk

    def _backup_create(self, **data):
        """兜底模型映射
        {
            "backup-gpt-4": "deepseek"
        }
        """
        backup_data = data.copy()
        backup_data['model'] = "backup-gpt-4" if data['model'].startswith('gpt-4') else "backup-gpt"  # todo: 4v
        backup_response = self.backup_client.chat.completions.create(**backup_data)

        send_message(f"入参：{data}")  # 兜底监控

        if data.get('stream'):
            def gen():
                for chunk in backup_response:
                    chunk.model = data['model']
                    yield chunk

            return gen()
        else:
            backup_response.model = data['model']

        return backup_response

    def _handle_error(self, data: Dict[str, Any], error: Union[str, Exception]) -> Any:
        """
        Handle errors and return an appropriate response.
        """
        if data.get('stream'):
            # Assuming chat_completion_chunk_error is defined elsewhere
            chat_completion_chunk_error.choices[0].delta.content = str(error)
            return chat_completion_chunk_error
        else:
            # Assuming chat_completion_error is defined elsewhere
            chat_completion_error.choices[0].message.content = str(error)
            return chat_completion_error

    @staticmethod
    def check_model(request: ChatCompletionRequest):

        # 直接走官转
        if request.model.startswith(("gpt-4",)):
            if (any((request.response_format, request.function_call))
                or (request.messages and "\n\nThe APIs you can use:\n\n" in request.messages[0].get('content', ''))
            ):
                return True


if __name__ == '__main__':
    from chatllm.llmchain.completions import github_copilot

    data = {
        'model': 'gpt-4',
        'messages': [
            {'role': 'system', 'content': "你是gpt4, Let's think things through one step at a time."},
            {'role': 'user', 'content': '你好'}
        ],
        'stream': False
    }

    # for i in range(3):
    #     print(Completions().create(ChatCompletionRequest(**data)))
    #     break

    data['stream'] = True

    api_key = github_copilot.Completions.get_access_token('xx')
    print(api_key)
    for i in Completions(api_key=api_key).create(ChatCompletionRequest(**data)):
        print(i)

    # todo: 兜底有问题 。。。。。。。。。。。。。。

"""
CompletionsClient 1 failed: Error code: 401 - {'error': {'message': '无效的令牌 (request id: 20240219192730485852794Bkw1irIR)', 'type': 'new_api_error'}}
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : config
# @Time         : 2023/5/26 13:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import pandas as pd

from meutils.pipe import *
from meutils.db import MySQL
from meutils.decorators import clear_cuda_cache

from chatllm.llms import load_llm4chat

torch_gc = clear_cuda_cache(lambda: logger.info('Clear GPU'), bins=os.getenv('TIME_INTERVAL', 15))

######################配置#####################################
debug = os.getenv('DEBUG')

tokens = set(os.getenv('TOKENS', 'chatllm').split(','))
llm_model = os.getenv('LLM_MODEL', '')
embedding_model = os.getenv('EMBEDDING_MODEL')
device = os.getenv('DEVICE', 'cpu')
num_gpus = int(os.getenv('NUM_GPUS', 2))

llm_role = os.getenv('LLM_ROLE', '')

# 落库
db_url = os.getenv('DB_URL', '')
table_name = os.getenv('TABLE_NAME', 'llm')
###############################################################


if embedding_model:
    from sentence_transformers import SentenceTransformer

    embedding_model = SentenceTransformer(embedding_model)
else:
    class RandomSentenceTransformer:
        def encode(self, texts):
            logger.error("请配置 EMBEDDING_MODEL")
            return np.random.random((len(texts), 64))


    embedding_model = RandomSentenceTransformer()

# 获取 do_chat
_do_chat = load_llm4chat(model_name_or_path=llm_model, device=device, num_gpus=num_gpus)


def do_chat(query, **kwargs):
    if llm_role:
        query = """{role}\n请回答以下问题\n{question}""".format(question=query, role=llm_role)  # 增加角色扮演
    return _do_chat(query, **kwargs)


# 入库
def do_db(df: pd.DataFrame, table_name: str):
    try:
        import emoji
        df = df.astype(str)
        df['choices'] = df['choices'].astype(str).map(emoji.demojize)  # todo: 更优雅的解决方案

        if db_url:
            db = MySQL(db_url)
            db.create_table_or_upsert(df, table_name)
            if debug:
                logger.debug("Data written successfully 👍")
    except Exception as e:
        logger.error(f"Failed to write data ⚠️: {e}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : barkup
# @Time         : 2024/1/9 09:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/openai/openai-python
# todo: 设计更加通用的兜底方案【首先得有靠谱的渠道（多个兜底渠道，作双兜底）】
# 3.5 deepseek
# 遇到内容审核【只要抛错就走备用】
# 完全兼容openai 去掉无关字段

from meutils.pipe import *
from meutils.notice.feishu import send_message

from openai import OpenAI
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error, completion_keys
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

send_message = partial(
    send_message,
    title="ChatCompletion主备",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


@lru_cache()
class Completions(object):
    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None, **kwargs):
        self.clients = []

        # 需要兜底的模型
        params = dict(
            api_key=api_key or 'sk-...',
            base_url=base_url or 'https://api.githubcopilot.com',
            default_headers={'Editor-Version': 'vscode/1.85.1'},
            timeout=300,
        )
        # 如果这一步就报错呢
        self.client = OpenAI(**params)  # todo: 异步

        # todo: 一主多备
        self.backup_client = OpenAI(
            api_key=os.getenv("BACKUP_API_KEY"),
            base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1"),  # todo: 避免自己调用自己
            timeout=300,
        )

    def create(self, request: ChatCompletionRequest, **kwargs):
        # data = request.model_dump()
        # data = {key: data.get(key) for key in completion_keys if key in data}  # 去除多余key， 可能最新的0125不支持
        data = to_openai_completion_params(request)

        # 直接走官转：function_call, response_format 等等
        if self.check_model(request):
            send_message(f"直接走官转：{data}")
            return self._backup_create(**data)

        creates = [self.client.chat.completions.create, self._backup_create, ]
        e = None
        for i, _create in enumerate(creates):
            try:
                response = _create(**data)  # 尝试执行操作

                # break  # 如果操作成功，则跳出循环
                if i == 0 and data.get('stream'):
                    return self.post_process(response, data)

                if data.get('stream'):
                    return response
                else:
                    n = 1.2
                    response.usage.total_tokens = int(response.usage.total_tokens * n)
                    response.usage.prompt_tokens = int(response.usage.prompt_tokens * n)
                    response.usage.completion_tokens = int(response.usage.completion_tokens * n)
                    return response

            except Exception as e:  # 走兜底
                _ = f"CompletionsClient {i} failed: {e}"  # rate limit exceeded：阶段性跳过这样的请求，防止被封
                send_message(_)
                logging.error(_)
        return self._handle_error(data, e)  # UnboundLocalError: local variable 'e' referenced before assignment

    def post_process(self, response, data):  # copilot
        """兜底判断"""
        for chunk in response:
            if chunk.choices:
                if chunk.choices[0].finish_reason == 'content_filter':  # 走兜底
                    _ = f"ContentFilter：{chunk.model_dump_json()}"
                    logger.debug(_)
                    send_message(_)
                    yield from self._backup_create(**data)
                    break

                if chunk.choices[0].delta.content or chunk.choices[0].finish_reason:
                    yield chunk

    def _backup_create(self, **data):
        backup_data = data.copy()
        backup_data['model'] = "backup-gpt-4" if data['model'].startswith('gpt-4') else "backup-gpt"  # todo: 4v
        backup_response = self.backup_client.chat.completions.create(**backup_data)

        send_message(f"入参：{data}")  # 兜底监控

        if data.get('stream'):
            def gen():
                for chunk in backup_response:
                    chunk.model = data['model']
                    yield chunk

            return gen()
        else:
            backup_response.model = data['model']

        return backup_response

    def _handle_error(self, data: Dict[str, Any], error: Union[str, Exception]) -> Any:
        """
        Handle errors and return an appropriate response.
        """
        if data.get('stream'):
            # Assuming chat_completion_chunk_error is defined elsewhere
            chat_completion_chunk_error.choices[0].delta.content = str(error)
            return chat_completion_chunk_error
        else:
            # Assuming chat_completion_error is defined elsewhere
            chat_completion_error.choices[0].message.content = str(error)
            return chat_completion_error

    @staticmethod
    def check_model(request: ChatCompletionRequest):

        # 直接走官转
        if request.model.startswith(("gpt-4",)):
            if (any((request.response_format, request.function_call))
                or (request.messages and "\n\nThe APIs you can use:\n\n" in request.messages[0].get('content', ''))
            ):
                return True

    @property
    def backup_llm_map(self):
        """兜底模型映射"""
        return {
            "backup": "deepseek",  # 兜底
            "backup-gpt": "deepseek",
            "backup-gpt-openai": "gpt-3.5-turbo-0125",

            "backup-gpt-4": "gpt-4-0125-preview",
            "backup-gpt-4-openai": "gpt-4-0125-preview",

            "backup-text-embedding-ada-002": "text-embedding-ada-002"
        }


if __name__ == '__main__':
    from chatllm.llmchain.completions import github_copilot

    data = {
        'model': 'gpt-4',
        'messages': [
            {'role': 'system', 'content': "你是gpt4, Let's think things through one step at a time."},
            {'role': 'user', 'content': '1+1'}
        ],
        'stream': False
    }

    # data = {'messages': [{
    #                   'content': '## Tools\n\nYou can use these tools below:\n\n### Clock Time\n\nDisplay a clock to show current time\n\nThe APIs you can use:\n\n#### clock-time_getCurrentTime_standalone\n\n获取当前时间\n\n### Realtime Weather\n\nGet realtime weather information\n\nThe APIs you can use:\n\n#### realtime-weather_fetchCurrentWeather\n\n获取当前天气情况\n\n### DALL·E 3\n\nWhenever a description of an image is given, use lobe-image-designer to create the images and then summarize the prompts used to generate the images in plain text. If the user does not ask for a specific number of images, default to creating four captions to send to lobe-image-designer that are written to be as diverse as possible.\n\n  All captions sent to lobe-image-designer must abide by the following policies:\n\n1. If the description is not in English, then translate it.\n2. Do not create more than 4 images, even if the user requests more.\n3. Don\'t create images of politicians or other public figures. Recommend other ideas instead.\n5. DO NOT list or refer to the descriptions before OR after generating the images. They should ONLY ever be written out ONCE, in the prompts field of the request. You do not need to ask for permission to generate, just do it!\n6. Always mention the image type (photo, oil painting, watercolor painting, illustration, cartoon, drawing, vector, render, etc.) at the beginning of the caption. Unless the caption suggests otherwise, make at least 1--2 of the 4 images photos.\n7. Diversify depictions of ALL images with people to include DESCENT and GENDER for EACH person using direct terms. Adjust only human descriptions.\n  - EXPLICITLY specify these attributes, not abstractly reference them.  The attributes should be specified in a minimal way and should directly describe their physical form.\n  - Your choices should be grounded in reality. For example, all of a given OCCUPATION should not be the same gender or race. Additionally, focus on creating diverse, inclusive, and exploratory scenes via the properties you choose during rewrites.  Make choices that may be insightful or unique sometimes.\n  - Use "various" or "diverse" ONLY IF the description refers to groups of more than 3 people. Do not change the number of people requested in the original description.\n  - Don\'t alter memes, fictional character origins, or unseen people. Maintain the original prompt\'s intent and prioritize quality.\n  - Do not create any imagery that would be offensive.\n\n8. Silently modify descriptions that include names or hints or references of specific people or celebrities by carefully selecting a few minimal modifications to substitute references to the people with generic descriptions that don\'t divulge any information about their identities, except for their genders and physiques. Do this EVEN WHEN the instructions ask for the prompt to not be changed. Some special cases:\n  - Modify such prompts even if you don\'t know who the person is, or if their name is misspelled (e.g. "Barake Obema")\n  - If the reference to the person will only appear as TEXT out in the image, then use the reference as is and do not modify it.\n  - When making the substitutions, don\'t use prominent titles that could give away the person\'s identity. E.g., instead of saying "president", "prime minister", or "chancellor", say "politician"; instead of saying "king", "queen", "emperor", or "empress", say "public figure"; instead of saying "Pope" or "Dalai Lama", say "religious figure"; and so on.\n  - If any creative professional or studio is named, substitute the name with a description of their style that does not reference any specific people, or delete the reference if they are unknown. DO NOT refer to the artist or studio\'s style.\n\nThe prompt must intricately describe every part of the image in concrete, objective detail. THINK about what the end goal of the description is, and extrapolate that to what would make satisfying images.\nAll descriptions sent to lobe-image-designer should be a paragraph of text that is extremely descriptive and detailed. Each should be more than 3 sentences long.\n\nThe APIs you can use:\n\n#### lobe-image-designer_text2image____builtin\n\nCreate images from a text-only prompt.',
    #                   'role': 'system'}, {'content': '现在几点\n\n', 'role': 'user'}], 'model': 'gpt-4',
    #  'frequency_penalty': 0.0, 'function_call': None, 'max_tokens': None, 'n': 1, 'presence_penalty': 0.0,
    #  'response_format': None, 'stop': None, 'stream': True, 'temperature': 0.0, 'top_p': 1.0, 'user': None}

    # for i in range(3):
    #     print(Completions().create(ChatCompletionRequest(**data)))
    #     break

    data['stream'] = True

    api_key = github_copilot.Completions.get_access_token(None)
    print(api_key)
    for i in Completions(api_key=api_key).create(ChatCompletionRequest(**data)):
        print(i)

    # todo: 兜底有问题 。。。。。。。。。。。。。。

# ChatCompletionChunk(id='chatcmpl-8uXP8LrO95xl1UPNKyEdZ1wgOqsQn', choices=[Choice(delta=ChoiceDelta(content=None, function_call=None, role=None, tool_calls=None), finish_reason='stop', index=0, logprobs=None)], created=1708485466, model='gpt-4', object='chat.completion.chunk', system_fingerprint='fp_f084bcfc79')
# ChatCompletionChunk(id='chatcmpl-8uXPvLSidJJSeM9IBPqjSqvuo4GZL', choices=[Choice(delta=ChoiceDelta(content=None, function_call=None, role=None, tool_calls=None), finish_reason='stop', index=0, logprobs=None, content_filter_offsets={'check_offset': 118, 'start_offset': 118, 'end_offset': 141}, content_filter_results={'hate': {'filtered': False, 'severity': 'safe'}, 'self_harm': {'filtered': False, 'severity': 'safe'}, 'sexual': {'filtered': False, 'severity': 'safe'}, 'violence': {'filtered': False, 'severity': 'safe'}})], created=1708485515, model=None, object=None, system_fingerprint=None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : models
# @Time         : 2023/7/31 17:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from fastapi import APIRouter, Body, Depends, HTTPException
from chatllm.serve.routes.utils import check_api_key
from chatllm.schemas.openai_api_protocol import *

router = APIRouter()

models = []


@router.get("/v1/models", dependencies=[Depends(check_api_key)])
async def show_available_models():
    # TODO: return real model permission details
    model_cards = []
    for m in models:
        model_cards.append(ModelCard(id=m, root=m, permission=[ModelPermission()]))
    return ModelList(data=model_cards)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : groq
# @Time         : 2024/2/20 16:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import requests

from meutils.pipe import *

url = "https://api.groq.com/v1/request_manager/text_completion"
data = {
    "system_prompt": "Please try to provide useful, helpful and actionable answers.",
    "user_prompt": "1+1",
    "model_id": "mixtral-8x7b-32768",
    # "history": [{"user_prompt": "讲个故事", "assistant_response": "好的，我来讲一个关于"}],
    "seed": 10,
    "max_tokens": 32768,
    "temperature": 0.2,
    "top_k": 40, "top_p": 0.8,
    "max_input_tokens": 21845
}
access_token = """
eyJhbGciOiJSUzI1NiIsImtpZCI6IjU1YzE4OGE4MzU0NmZjMTg4ZTUxNTc2YmE3MjgzNmUwNjAwZThiNzMiLCJ0eXAiOiJKV1QifQ.eyJhdWQiOiI5NTIwNjQ0MjA1OTAtc2ZiOG81NGxlcWQwZGViczZyZmNycWJ1ZmU0M2U3c2suYXBwcy5nb29nbGV1c2VyY29udGVudC5jb20iLCJhenAiOiJhbm9ueW1vdXMtYXBpLXVzZXJAZ3JvcS1jbG91ZC1zdGFnaW5nLmlhbS5nc2VydmljZWFjY291bnQuY29tIiwiZW1haWwiOiJhbm9ueW1vdXMtYXBpLXVzZXJAZ3JvcS1jbG91ZC1zdGFnaW5nLmlhbS5nc2VydmljZWFjY291bnQuY29tIiwiZW1haWxfdmVyaWZpZWQiOnRydWUsImV4cCI6MTcwODQyMDA0MSwiaWF0IjoxNzA4NDE2NDQxLCJpc3MiOiJodHRwczovL2FjY291bnRzLmdvb2dsZS5jb20iLCJzdWIiOiIxMTY4MjY2OTYyNzQ1NzAzNTg4NTcifQ.flEPxLXy4mYnrJwHofP-cd8I9HlXU1NOfHdgaQUPRj-2AEFsNFqouZB3HsrpqijoC1xeGpf7yKDfc1ryHvANHCDN1lzptkrI_rG3gg17Y_naqP-OKiUddYXXQEXAvSIOBlWRxO_NFjrhrbZCI9nXpUpcXfHSlOIluYZaqYbrJEAtgeeWjpFKRCWW1SVeorTWBrgO27_hOOy2AbvbLd7VKWPgjdii28jaF7t_O2PqHINzD7DMSXeDhbXaouKwhi7QqBChbmS8Q-K05VwEg6T9rQyaUVLH1RleW2n2tzRynjTOA91uj_XAo0Ol0o7GHyUWhNHFC-Fr92fjuGyBL83RCQ
""".strip()

headers = {
    'Authorization': f"Bearer {access_token}",  # access_token
    'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) '
                  'Chrome/119.0.0.0 Safari/537.36'
}

chunks = requests.post(url, json=data)
print(chunks.text)
#
# for i in chunks.iter_lines():
#     print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st_chat
# @Time         : 2023/8/11 14:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

import streamlit as st

from meutils.pipe import *

from chatllm.llmchain import init_cache
from chatllm.llmchain.applications import ChatFile
from chatllm.llmchain.document_loaders import FileLoader
from chatllm.llmchain.embeddings import OpenAIEmbeddings
from chatllm.llmchain.prompts.prompt_templates import CHAT_CONTEXT_PROMPT_WITH_SOURCE

init_cache(1)

context_prompt_template = """
根据以下信息，简洁、专业地回答用户的问题。如果无法得到答案，请回复：“根据已知信息无法回答该问题”或“没有提供足够的信息”。请勿编造信息，答案必须使用中文。

已知信息：
{context}

问题：
{question}

让我们一步一步思考并回答：
""".strip()  # Let's think step by step


class ChatMessage(BaseModel):
    role: str
    content: str


def chat(
    user_role='user',

    assistant_role='assistant',
    assistant_avator="nesc.jpeg",

    reply_func=lambda input: f'{input}的答案',
    max_turns=3,
):
    def chat_message(role):
        if role == 'user':
            return st.chat_message(user_role)
        else:
            return st.chat_message(assistant_role, avatar=assistant_avator)

    # Initialize chat history
    if "messages" not in st.session_state:
        st.session_state.messages = []
    else:
        st.session_state.messages = st.session_state.messages[-2 * (max_turns - 1):]

    # Display chat messages from history on app rerun
    for message in st.session_state.messages:
        with chat_message(message.role):
            st.markdown(message.content, unsafe_allow_html=True)

    prompt = st.chat_input("    🔥请提问？")
    if prompt:
        print('\n')
        print(prompt)
        prompt = prompt.strip()
        # Display user message in chat message container
        with chat_message(user_role):
            st.markdown(prompt)
        # Add user message to chat history
        st.session_state.messages.append(ChatMessage(role=user_role, content=prompt))

        with chat_message(assistant_role):
            message_placeholder = st.empty()

            response = ''
            gen = reply_func(prompt) or '根据已知信息无法召回相关内容。'
            for token in gen:
                # Display robot response in chat message container
                response += token
                message_placeholder.markdown(response + "▌")
            message_placeholder.markdown(response, unsafe_allow_html=True)

        # Add robot response to chat history
        st.session_state.messages.append(ChatMessage(role=assistant_role, content=response))


@st.cache_resource
def get_reply_func(file):
    if file:
        docs = FileLoader(file, file.name).load_and_split()
        print(file.name, len(docs))
        from chatllm.llmchain.vectorstores import Milvus, FAISS

        cb = ChatFile(embeddings=OpenAIEmbeddings(chunk_size=20), prompt_template=context_prompt_template)
        cb.create_index(docs)

        reply_func = lambda query: cb.llm_qa(query=query, k=5, threshold=0.5)

        return reply_func

    source = """
        <details markdown="1">
            <summary>详情</summary>

    - [ ] 功能点
        - [x] 接入非结构化文档（已支持 pdf、docx 文件格式）
        - [ ] 增加多级缓存缓存

        </details>
        """.strip()

    reply_func = lambda input: f'### ⚠️请先上传文档\n' + source
    return reply_func


if __name__ == '__main__':
    st.markdown('# 📔基于本地知识库问答')

    file = st.file_uploader("Choose a file", type=['pdf', 'doc', 'docx', 'txt', ], help='目前仅支持单文档问答')
    print(f"{time.ctime()}: {file}")
    with st.spinner('AI正在处理...'):
        reply_func = get_reply_func(file)

        chat(reply_func=reply_func)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2023/5/19 17:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

import torch
from transformers import AutoTokenizer, AutoModel, LlamaForCausalLM


class LLM(object):

    def __init__(self, model_name_or_path="THUDM/chatglm-6b", device='cpu', max_num_gpus=2):
        self.model_name_or_path = model_name_or_path
        self.device = device
        self.max_num_gpus = max_num_gpus

        self.model, self.tokenizer = self.load()

    def load(self):
        tokenizer = AutoTokenizer.from_pretrained(self.model_name_or_path, trust_remote_code=True)

        if 'llama' in self.model_name_or_path.lower():  # llama 系列
            model = LlamaForCausalLM.from_pretrained(self.model_name_or_path, trust_remote_code=True)
        else:
            model = AutoModel.from_pretrained(self.model_name_or_path, trust_remote_code=True)

        if torch.cuda.is_available() and self.device.lower().startswith("cuda"):
            num_gpus = min(self.max_num_gpus, torch.cuda.device_count())

            if num_gpus == 1:  # 单卡
                model = model.half().cuda()
            else:
                pass  # todo: 多卡
        else:
            model = model.float().to(self.device)

        return model.eval(), tokenizer

    def chat(self):
        return partial(self.model.stream_chat, tokenizer=self.tokenizer)  # 思考 统一模式



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 逆向工程

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from sse_starlette import EventSourceResponse
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

send_message = lru_cache(send_message)


@router.post("/chat/completions")
async def create_chat_completions_for_rag(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    request.model = request.model[4:]  # rag-gpt-4、rag-gpt-4-file-id
    if '-file-' in request.model:  # gpt-4-file-id
        request.model, *request.file_ids = request.model.split('-file-')

    client = OpenAI(
        api_key=api_key,
        base_url='https://api.chatllm.vip/v1',
        http_client=httpx.Client(follow_redirects=True)
    )
    if request.file_ids:
        file_content = request.file_ids | xThreadPoolExecutor(
            lambda file_id: client.files.content(file_id).text) | xjoin('\n')

        request.messages = [
                               {
                                   "role": "system",
                                   "content": "你是 ChatfireBot，由 Chatfire AI 提供的人工智能助手，你更擅长中文和英文的对话。你会为用户提供安全，有帮助，准确的回答。同时，你会拒绝一些涉及恐怖主义，种族歧视，黄色暴力等问题的回答。Chatfire AI 为专有名词，不可翻译成其他语言。",
                               },
                               {
                                   "role": "system",
                                   "content": file_content,
                               }
                           ] + request.messages

    data = to_openai_completion_params(request)
    response = client.chat.completions.create(**data)

    return openai_response2sse(response)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatgpt
# @Time         : 2023/6/29 08:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://api2d-doc.apifox.cn/api-84787447

from meutils.pipe import *


def load_llm4chat(**kwargs):
    import openai

    def stream_chat(query, history=None, **chat_kwargs):
        history = history or []
        messages = history + [{"role": "user", "content": query}]

        kwargs = {
            "model": "gpt-3.5-turbo-0613",
            "stream": True,
            "max_tokens": None,
            "temperature": None,
            "top_p": None,
            "messages": messages,
            "user": "Betterme"
        }
        chat_kwargs = {**kwargs, **chat_kwargs}
        chat_kwargs = {k: chat_kwargs[k] for k in kwargs}  # 过滤不支持的参数

        completion = openai.ChatCompletion.create(**chat_kwargs)

        for c in completion:
            _ = c.choices[0].get('delta').get('content', '')
            yield _

    return stream_chat


if __name__ == '__main__':
    pass
